#include "RNACons.h"

MSA::MSA(const string & infile) {
	if(!fexists(infile.c_str())) {
		cerr<<"File "<<infile<<" does not exist. "<<endl;
		exit(0);
	}

	bool isfasta = true;//by default, fasta;
	ifstream fin(infile.c_str());
	string str1; 

	readin(fin, str1);
	str1 = lowercase(str1);
	if(str1.find("clustal")!=string::npos) {
		isfasta = false;
	} else if (str1.find(">")==0) {
		//isfasta
	} else {
		cerr<<"Can not recogize format of the file"<<endl;
		exit(0);
	}
	fin.close();

	if (isfasta) readfasta(infile);
	else readclustalw(infile);
}

MSA::MSA(const MSA & in_msa) {
	N = in_msa.N; 
	L = in_msa.L;
	names = in_msa.names; 
	species = in_msa.species;
}

string MSA::operator [] (const int & i) {
	if(i>=0 && i<sz(names)) {
		return species[names[i]];
	} else {
		cerr<<"There are "<<sz(names)<<" sequences in the multiple sequence alignment."<<endl;
		cerr<<"Can not find sequence of the "<<i<<"-th species in the multiple sequence alignment."<<endl;
		exit(0);
	}
}

void MSA::readfasta(const string & infile) {
	ifstream fin(infile.c_str());
	string s;
	int mode = 0; //0 - reading comments 
	//1 - reading name
	//2 - reading sequence 

	int line = 0;
	string name = "";
	N = L = 0; 
	while(!fin.eof()) {
		line++;
		readin(fin, s);
		s = trim(s);
		if(s.empty() || (!s.empty() && s[0]=='#')) {
			mode = 0;
			continue;
		} else if(s[0]=='>'){
			name = s.substr(1);
			names.PB(name);
			N++;
			mode = 1;
		} else {
			s = dna2rna(s);
			if(mode == 1) {
				species[name] = s;
				mode = 2;
			} else if(mode==2) {
				species[name] += s;
			} else if(mode==0) {
				cerr<<"An error encoutered when processing line "<<line<<". The program tried to read a fasta (multiple sequence alignment) file, and could not determine the species of the current sequence."<<endl;
				exit(0);
			}
		}
	}
	fin.close();
	
	if(N!=0) L = sz(species[names[0]]); 
}


void MSA::readclustalw(const string & infile) {
	ifstream fin(infile.c_str());
	string s; 
	int line = 0;
	N = L = 0; 
	while(!fin.eof()) {
		line++;
		readin(fin, s);
//		cout<<"line="<<line<<": "<<s<<endl;
		s = trim(s);
		string ls = lowercase(s);
		if(s.empty() || (!s.empty() && s[0]=='#') || s.find("*") !=string::npos) {
			continue;
		} else if(ls.find("clustal")!=string::npos) {
			continue;
		} else {		
			string name, seq; 
			parseclustalline(s, name, seq);
			seq = dna2rna(seq);
			
			map<string,string>::iterator it;
			it = species.find(name);
			if(it == species.end()) {
				species[name] = seq;
				names.PB(name);
				N++;
			} else {
				species[name] += seq;
			}
		}
	}
	fin.close();

	if(N!=0) L = sz(species[names[0]]); 	
	if(!checkformat()) {
		exit(0);
	}
}


void MSA::show(ostream & fout) {
	FOR(i,0,sz(names)) {
		fout<<">"<<names[i]<<"\n"<<species[names[i]]<<endl;
	}
}

bool MSA::isempty() {
	if(N == 0) return true;
	if(L == 0) return true;
	return false;
}

bool MSA::checkformat() {
	int length = 0;
	FOR(i,0,N) {
		if(i==0) length = sz(species[names[i]]);
		if(sz(species[names[i]]) != length) {
			cerr<<"The sequence length of ["<<names[i]<<"] is different from those of the others. "<<endl;
			return false;
		}
	}
	return true;
}
//Given a species name, return its index in the msa.
int MSA::name2indx(const string & name) {
	FOR(i,0,sz(names))	 {
		if(names[i].compare(name)==0) {
			return i; 
		}
	}
	return -1;
}

//remove gappy columns (whose gap percentage > p_maxgap) from the msa, and return the gappy-to-nongappy vector
//0 - this column has >= p_maxgap gaps and should be removed
//1 - this column has <  p_maxgap gaps and should be preserved
vector<int> MSA::compute_gap2ngap(const double & p_maxgap) {
	vector<int> ret;
	FOR(j, 0, L) {
		int gapcount = 0;
		FOR(i, 0, N) {
			if(species[names[i]][j] == '-')
				gapcount++;
		}
		if(double(gapcount) >= (double)N*p_maxgap) {
			ret.PB(0);
		} else {
			ret.PB(1);
		}
	}
	return ret;	
}

//return a new MSA which is constructed by applying the gappy-to-nongappy vector to the current vector
MSA MSA::apply_gap2ngap(vector<int> & gap2ngap) {
	//gap2ngap[j]=0: this column has >= p_maxgap gaps and should be removed
	//gap2ngap[j]=1: this column has <  p_maxgap gaps and should be preserved
	MSA newmsa;
	newmsa.names = names;
	FOR(i, 0, N) {
		string newseq = "";
		string org_seq = species[names[i]];
		FOR(j, 0, L) {
			if(gap2ngap[j]==0) {
				//do nothing
			} else if(gap2ngap[j]==1) {
				newseq += org_seq[j];				
			} else assert(false);
		}
		newmsa.species[names[i]] = newseq;
		newmsa.L = sz(newseq);
	}
	newmsa.N = N;
	return newmsa;
}












